@@ -1,5 +1,5 @@
 /* Test of c-stack module.
-   Copyright (C) 2002, 2004, 2006, 2008-2016 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004, 2006, 2008-2017 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,8 +30,6 @@
 
 #include "macros.h"
 
-char *program_name;
-
 static volatile int *
 recurse_1 (volatile int n, volatile int *p)
 {
@@ -60,7 +58,6 @@ main (int argc, char **argv)
   setrlimit (RLIMIT_STACK, &rl);
 #endif
 
-  program_name = argv[0];
   if (c_stack_action (NULL) == 0)
     {
       if (1 < argc)
