@@ -1,7 +1,7 @@
 /* sdiff - side-by-side merge of file differences
 
    Copyright (C) 1992-1996, 1998, 2001-2002, 2004, 2006-2007, 2009-2013,
-   2015-2016 Free Software Foundation, Inc.
+   2015-2017 Free Software Foundation, Inc.
 
    This file is part of GNU DIFF.
 
@@ -26,6 +26,7 @@
 
 #include <c-stack.h>
 #include <dirname.h>
+#include "die.h"
 #include <error.h>
 #include <exitfail.h>
 #include <file-type.h>
@@ -155,9 +156,8 @@ try_help (char const *reason_msgid, char const *operand)
 {
   if (reason_msgid)
     error (0, 0, _(reason_msgid), operand);
-  error (EXIT_TROUBLE, 0, _("Try '%s --help' for more information."),
+  die (EXIT_TROUBLE, 0, _("Try '%s --help' for more information."),
 	 program_name);
-  abort ();
 }
 
 static void
@@ -918,10 +918,10 @@ edit (struct line_filter *left, char const *lname, lin lline, lin llen,
 		  cmd0 = 'q';
 		  break;
 		}
-	      /* Fall through.  */
+	      FALLTHROUGH;
 	    default:
 	      flush_line ();
-	      /* Fall through.  */
+	      FALLTHROUGH;
 	    case '\n':
 	      give_help ();
 	      continue;
@@ -967,14 +967,14 @@ edit (struct line_filter *left, char const *lname, lin lline, lin llen,
 	      case 'd':
 		if (llen)
 		  {
+		    printint l1 = lline;
+		    printint l2 = lline + llen - 1;
 		    if (llen == 1)
-		      fprintf (tmp, "--- %s %ld\n", lname, (long int) lline);
+		      fprintf (tmp, "--- %s %"pI"d\n", lname, l1);
 		    else
-		      fprintf (tmp, "--- %s %ld,%ld\n", lname,
-			       (long int) lline,
-			       (long int) (lline + llen - 1));
+		      fprintf (tmp, "--- %s %"pI"d,%"pI"d\n", lname, l1, l2);
 		  }
-		/* Fall through.  */
+		FALLTHROUGH;
 	      case '1': case 'b': case 'l':
 		lf_copy (left, llen, tmp);
 		break;
@@ -989,14 +989,14 @@ edit (struct line_filter *left, char const *lname, lin lline, lin llen,
 	      case 'd':
 		if (rlen)
 		  {
+		    printint l1 = rline;
+		    printint l2 = rline + rlen - 1;
 		    if (rlen == 1)
-		      fprintf (tmp, "+++ %s %ld\n", rname, (long int) rline);
+		      fprintf (tmp, "+++ %s %"pI"d\n", rname, l1);
 		    else
-		      fprintf (tmp, "+++ %s %ld,%ld\n", rname,
-			       (long int) rline,
-			       (long int) (rline + rlen - 1));
+		      fprintf (tmp, "+++ %s %"pI"d,%"pI"d\n", rname, l1, l2);
 		  }
-		/* Fall through.  */
+		FALLTHROUGH;
 	      case '2': case 'b': case 'r':
 		lf_copy (right, rlen, tmp);
 		break;
